@@ -32,9 +32,8 @@
 socklen_t addr_get_len(const struct sockaddr *sa);
 uint16_t addr_get_port(const struct sockaddr *sa);
 int addr_set_port(struct sockaddr *sa, uint16_t port);
-bool addr_is_any(struct sockaddr *sa);
-bool addr_is_local(struct sockaddr *sa);
-bool addr_is_temp_inet6(struct sockaddr *sa);
+bool addr_is_any(const struct sockaddr *sa);
+bool addr_is_local(const struct sockaddr *sa);
 bool addr_unmap_inet6_v4mapped(struct sockaddr *sa, socklen_t *len);
 bool addr_map_inet6_v4mapped(struct sockaddr_storage *ss, socklen_t *len);
 bool addr_is_equal(const struct sockaddr *a, const struct sockaddr *b, bool compare_ports);
@@ -47,6 +46,7 @@ typedef struct addr_record {
 } addr_record_t;
 
 int addr_resolve(const char *hostname, const char *service, addr_record_t *records, size_t count);
+bool addr_is_numeric_hostname(const char *hostname);
 
 bool addr_record_is_equal(const addr_record_t *a, const addr_record_t *b, bool compare_ports);
 int addr_record_to_string(const addr_record_t *record, char *buffer, size_t size);
