@@ -25,9 +25,9 @@
 #include "internals.hpp"
 #include "logcounter.hpp"
 #include "peerconnection.hpp"
+#include "processor.hpp"
 #include "rtp.hpp"
 #include "sctptransport.hpp"
-#include "threadpool.hpp"
 
 #if RTC_ENABLE_MEDIA
 #include "dtlssrtptransport.hpp"
@@ -371,14 +371,15 @@ void PeerConnection::closeTransports() {
 
 	// Initiate transport stop on the processor after closing the data channels
 	mProcessor.enqueue([self = shared_from_this(), transports = std::move(transports)]() {
-		ThreadPool::Instance().enqueue([transports = std::move(transports)]() mutable {
-			for (const auto &t : transports)
-				if (t)
-					t->stop();
+		TearDownProcessor::Instance().enqueue(
+		    [transports = std::move(transports), token = Init::Instance().token()]() mutable {
+			    for (const auto &t : transports)
+				    if (t)
+					    t->stop();
 
-			for (auto &t : transports)
-				t.reset();
-		});
+			    for (auto &t : transports)
+				    t.reset();
+		    });
 	});
 }
 
@@ -630,14 +631,14 @@ shared_ptr<DataChannel> PeerConnection::emplaceDataChannel(string label, DataCha
 			stream += 2;
 		}
 	}
-	// If the DataChannel is user-negotiated, do not negotiate it here
+	// If the DataChannel is user-negotiated, do not negotiate it in-band
 	auto channel =
 	    init.negotiated
 	        ? std::make_shared<DataChannel>(weak_from_this(), stream, std::move(label),
 	                                        std::move(init.protocol), std::move(init.reliability))
-	        : std::make_shared<NegotiatedDataChannel>(weak_from_this(), stream, std::move(label),
-	                                                  std::move(init.protocol),
-	                                                  std::move(init.reliability));
+	        : std::make_shared<OutgoingDataChannel>(weak_from_this(), stream, std::move(label),
+	                                                std::move(init.protocol),
+	                                                std::move(init.reliability));
 	mDataChannels.emplace(std::make_pair(stream, channel));
 	return channel;
 }
