@@ -29,11 +29,14 @@
 
 namespace rtc::impl {
 
-class TcpServer {
+class TcpServer final {
 public:
 	TcpServer(uint16_t port);
 	~TcpServer();
 
+	TcpServer(const TcpServer &other) = delete;
+	void operator=(const TcpServer &other) = delete;
+
 	shared_ptr<TcpTransport> accept();
 	void close();
 
