@@ -95,13 +95,10 @@ bool TcpTransport::send(message_ptr message) {
 	if (state() != State::Connected)
 		throw std::runtime_error("Connection is not open");
 
-	if (state() != State::Connected)
-		return false;
-
 	if (!message)
 		return trySendQueue();
 
-	PLOG_VERBOSE << "Send size=" << (message ? message->size() : 0);
+	PLOG_VERBOSE << "Send size=" << message->size();
 	return outgoing(message);
 }
 
